﻿-- Action Bar Profiles
ABS = LibStub("AceAddon-3.0"):NewAddon("ProfilesActionBars", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

function ABS:OnInitialize()
   ABSdb = LibStub("AceDB-3.0"):New("ProfilesActionBars",nil,"Default")
end


function ABS:Echo(text)
   DEFAULT_CHAT_FRAME:AddMessage("Profiles: Action bars - "..text)
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function ABS:PreSaveABProfile(name)
	ABS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function ABS:SaveABSProfile(name)
	ABSdb:SetProfile(name)

	-- Solving /mp save tank X 2
	if ABS:ExistsProfile(name) then ABSdb:ResetProfile(name) end
	
   print("saving!")
   
   -- Cache icon path -> ID
	local icons_table = {}

	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
	
	local _,numperchar = GetNumMacros()
	ABSdb.profile["numMacros"] = {numperchar}
   
	for actionID=1, 120 do
      
		local type_ab, id, subType, extraID = GetActionInfo(actionID)

		if type_ab and id then
			ABSdb.profile[actionID] = {type_ab, id}
			
            -- Save a macro
			-- TODO: Save the whole macro incase ABS is standalone
			if( type_ab == "macro" ) then 
				ABSdb.profile[actionID] = { type_ab, name }
			end
		end
   end
end

-- This is an alias for ProfilesHub
function ABS:Save(...)
	ABS:SaveABSProfile(...)
end

-- ProfilesHub: Send message to all slaves that we are now switching profile
function ABS:PreLoadABProfile(name)

	
	-- No such profile saved
	if not ABS:ExistsProfile(name) then ABS:Echo(name.." profile not found.") return end

	-- In combat!
	if InCombatLockdown() then ABS:Echo("Cannot change macros in combat!") return end

	ABS:SendAceMsg("LoadABSProfile",name,"load") -- ProfilesHub
end

-- This function should not be called directly.
-- Should only be called by slave event!

function ABS:LoadABSProfile(name)

	-- In combat!
	if InCombatLockdown() then ABS:Echo("Cannot change macros in combat!") return end

	ABSdb:SetProfile(name)

	ABS:Echo("Loading action from profile "..name.."!")
	
	-- Start fresh with nothing on the cursor
	ClearCursor()
	
	
	--Load Action bar profile!
	for index, keys in pairs(ABSdb.profile) do
		
		if index ~= "numMacros" then
		
			
			
			if keys[1] == "spell" 		 then PickupSpell(keys[2]) end
			
			if keys[1] == "equipmentset" then PickupEquipmentSetByName(keys[2]) end
			
			if keys[1] == "item"         then PickupItem(keys[2]) end
			
			if keys[1] == "macro"        then
				if not MPS then
					PickupMacro(keys[2])
					PlaceAction(index)
				else
					ClearCursor()
				end
			end
			
			
			PlaceAction(index)
		end
	end
	
	MPS:SendMessage("ABS_Profiles_load",name) -- ProfilesHub
end

-- This is an alias for ProfilesHub
function ABS:Load(...)
	ABS:LoadABSProfile(...)
end


function ABS:ExistsProfile(name)
   for i, k in ipairs(ABSdb:GetProfiles()) do
      local key = k
      
      if key == name then
         return (key)
      end
   end
   return nil
end


SLASH_ABS1 = "/ABS"
SlashCmdList["ABS"] = function(msg)
   local cmd, arg = string.split(" ", msg, 2)
   cmd = string.lower(cmd or "")
   arg = arg or ""
   
   if (cmd == "version") then
      print("Keyboard Layout Saver v."..KLSversion)
   elseif (cmd=="list")then
      for i,k in ipairs(ABSdb:GetProfiles()) do
		if k == ABSdb:GetCurrentProfile() then
			print(' - ' .. k.." (current)")
		else
			print(' - ' .. k)
		end
	end
   elseif (cmd=="" or arg=="")then
      ABS:Echo("/ABS delete <name>")
      ABS:Echo("/ABS save <name>")
      ABS:Echo("/ABS load <name>")
      ABS:Echo("/ABS list")
      ABS:Echo("/ABS version")
      ABS:Echo("/ABS debug [on|off]")
   elseif (cmd=="save" and arg~="")then
      ABS:PreSaveABProfile(arg)
   elseif (cmd=="load" and arg~="")then
      ABS:PreLoadABProfile(arg)
   elseif (cmd=="delete" and arg~="")then
      ABS:DeleteProfil(arg)
   
   else    
      ABS:Echo("wrong entry")
   end
end